% ----------------------------------------------------------------
% Code for nonsymmetric nonlinear diffusion/conduction problem
% This problem is discussed by:
%   Finlayson (1972), pp 16-19, "The Method of Weighted Residuals and Variational Principles"
%   also solved by Orth. Coll. at http://faculty.washington.edu/finlayso/ebook/bvp/OC/OCheat.htm
%   and finite difference method at http://faculty.washington.edu/finlayso/ebook/index/index.htm
%   We treat the problem using the methods described at http://tildentechnologies.com/Numerics/index.html
%   With Lobatto points the exact heat flux is calculated for 1 or more points
% Approx. Flux Left Flux Right y(0.5)     Max(error)
%  n=1      1.5      1.5      0.58333     1.1146e-02
%  n=2      1.5      1.5      0.58337     2.2348e-03  < y(0.5) interpolated
%  n=3      1.5      1.5      0.58102     4.8800e-04
%  n=4      1.5      1.5      0.58102     1.2273e-04  < y(0.5) interpolated
%  n=5      1.5      1.5      0.58115     3.3763e-05
% exact     1.5      1.5      0.58114
% ----------------------------------------------------------------
clear all;
addpath('mn');    % path to other files
run OCCdefs
function [D] = DiffC(y)  % diffusion coefficient
   D = 1.0 + y;
end
function [y] = Exact(x)  % exact solution
   y = sqrt(1 + 3.0*x) - 1.0;
end
nout = 9; xi = [0:1.0/(nout-1.0):1];  % interpolation points
result(1:4,1:nout) = 0.0;
result(1,:) = xi;
result(3,:) = Exact(xi);  % exact for comparison
symm = NonSymmetric; % default symmetry
meth = Lobatto;      % default point type
fout = 1;  fstd = 1;
epstol = 1.0e-8;  itmax = 20;
ncalc = [1,5];
% --------------------------------------------------------
for n=ncalc(1):ncalc(2)
   fprintf(fstd,'\nSolution using Lobatto points with %2i interior points\n',n);
   % --------------------------------------------------------
   nt = n + 2;  % total points
   [xc,wc,Ac] = OCnonsym(n,meth);         % .m meth 1:5 = Lobatto, Gauss, Chebyshev, Radau R, Radau L
   [Lc] = Lcoef(xi',xc);
   % --------------------------------------------------------
   rc(1:3,1:nt) = repmat(xc',3,1);
   y = xc;     % initialize solution
   for i = 1:itmax
      D = DiffC(y);
      C = (Ac')*(bsxfun(@times,(wc.*D),Ac));   % A'*W*A stiffness matrix
      f = (C*y)';  % 2:n+1 are residuals, 1 & n+2 are fluxes
      err = max(abs(f(2:n+1)));
      fprintf(fstd,' Iteration & error: %i %10.3e\n',i,err);
      if(err < epstol)
         break;
      end
      y(2:n+1) = -C(2:n+1,2:n+1)\C(2:n+1,nt);
   end
   y(2:n+1) = -C(2:n+1,2:n+1)\C(2:n+1,nt);
   disp('solution: xc, yc:');
   disp('last line: left flux, interior residuals, right flux:');
   rc(2,:) = y';
   rc(3,:) = f
   disp('Interpolated solution: x, y, exact, error:');
   result(2,:) = (Lc*y)';  % interpolated solution
   result(4,:) = result(2,:) - result(3,:)  % interpolated solution
   errmx(n) = max(abs(result(4,:)));
end  % for ncalc
disp('Max error for each solution:');
errmx

