function [ys,yc] = DIRK(A,wx,Cx,dz,method,nstep)
   % ------------------------------------
   % Diagonally Implicit Runge-Kutta - set up for a general method
   % ns - number of stages
   % method - method designator
   % DirkParm - provides the parameters given the method
   % Explicit1 - .true. for explicit 1st stage
   % ------------------------------------
   nstage = [2,3,3,4,5];        % no. stages
   ns = nstage(method);
   n = size(A,1);
   yc(1:nstep,1:n+2) = 0.0;  ys(1:nstep,1:3) = 0.0;
   f(1:n,1:ns) = 0.0;  r(1:n) = 0.0;
   [ar,br,cr] = DirkParm(method);
   Explicit1 = max(ar(1,:)) == 0.0;
   ar = ar*dz; br = br*dz; cr = cr*dz;
   ad = ar(ns,ns); adi = 1.0/ad;
   wa = wx/ad;  wi = (-1.0)./wx;
   y(1:n,1) = 1.0;
   [ys(1,:),yc(1,:)] = CalcFlux(y,wx,Cx);
   Ax = A + diag(wa);
   [L,U] = lu(Ax);   % LU factors
   for k = 2:nstep
      y0 = y;
      if(Explicit1)
         f(:,1) = (wi').*(A*y);
      else
        %f(:,1) = (Ax\((wa').*y) - y)*adi;    % refactors every time
         f(:,1) = (U\(L\((wa').*y)) - y)*adi; % use factors
      end
      for i = 2:ns % loop over implicit stages
         y = y0 .+ f(:,1:i-1)*(ar(i,1:i-1)');
         f(:,i) = (U\(L\((wa').*y)) - y)*adi;
      end
      y = y0 .+ f*br';
      [ys(k,:),yc(k,:)] = CalcFlux(y,wx,Cx);
   end
end
% ------------------------------------
function [a,b,c] = DirkParm(method)   % returns Butcher tableu
global fout symm geom meth         % output file handle
   NDIRKS = 5;   % number methods stored
   sqr2 = sqrt(2.0);
   % Parameters for ESDIRK2(1)3L[2]SA (2 implicit stages)
   E3g = 1.0 - (1.0)/sqr2;
   % Parameters for ESDIRK3(2)4L[2]SA (3 implicit stages)
   E4g = (0.43586652150845899941601945);
   E4c3 = 0.6;  E4xx = E4c3-2.0*E4g;   E4a32 = E4c3*E4xx/(4.0*E4g);
   E4b2 = (-2.0 + 3.0*E4c3+6.0*E4g*(1.0-E4c3))/(12.0*E4g*E4xx);
   E4b3 =(1.0 - 6.0*E4g+6.0*E4g**2)/(3.0*E4c3*E4xx);
   % Parameters for ESDIRK3(2)5L[2]SA (4 implicit stages)
   E5g = (9.0)/(40.0);
   E5c(1:5) = [0.0, (2.0*E5g), (2.0 + sqr2)*E5g, 0.6, 1.0];
   E5a43 = (-7.0)/((1.0+sqr2)*40.0);    E5a54 = (5827.0)/(7560.0);
   E5a53 =(-2374.0)*(1.0 + 2.0*sqr2)/(2835.0*(5.0 + 3.0*sqr2));
   % Method diagonals
   Dg(1:NDIRKS) = [E3g,E3g,E4g,E4g,E5g];  % diagonals
   first(1:NDIRKS) = [1,2,1,2,2];         % 1st implicit stage
   nstage(1:NDIRKS) = [2,3,3,4,5];        % no. stages
   % ------------------------------------------------------------------------
   ns = nstage(method);   a(1:ns,1:ns) = 0.0;
   i = first(method);     a(i:ns,i:ns) = Dg(method)*eye(ns-i+1);
   switch method
   case 1  % --- 2nd order 2 stage SDIRK (p. 72 Nasa review, Alexander 1977)
      a(2,1) = 1.0 - E3g;
   case 2  % --- 2nd order 3 stage ESDIRK2(1)3L[2]SA (p. 73 Nasa review)
      a(3,1) = (0.25 - 0.50*E3g)/E3g;
      a(2,1) = E3g;  a(3,2) = a(3,1);
   case 3  % --- 3rd order 3 stage SDIRK (p. 77 Nasa review, Alexander 1977)
      a(3,2) = 1.25 + E4g*((1.5)*E4g - 5.0);
      a(3,1) = 1.0 - a(3,2) - a(3,3);  a(2,1) = (1.0 - E4g)*(0.5);
   case 4  % --- 3rd order 4 stage ESDIRK3(2)4L[2]SA (p. 75 Nasa review)
      a(2,1) = E4g;  a(3,2) = E4a32; a(3,1) = E4c3 - sum(a(3,2:3));
      a(4,3) = E4b3; a(4,2) = E4b2;  a(4,1) = 1.0 - sum(a(4,2:4));
   case 5  % --- 3rd order 5 stage ESDIRK3(2)5L[2]SA (superconvergent?)
      a(2,1) = E5g; a(4,3) = E5a43; a(5,4) = E5a54;  a(5,3) = E5a53;
      for i=3:5
         a(i,1:2) = (E5c(i) - sum(a(i,3:i)))*0.5;
      end
   end
   b = a(ns,:);  % stiffly accurate (make exception if not true)
   for i = 1:ns
      c(i) = sum(a(i,:));   % c = row sum(a)
   end
 %  ArrayPrint(fout,a,'Dirk a')
 %  ArrayPrint(fout,b,'Dirk b')
 %  ArrayPrint(fout,c,'Dirk c')
end %function DirkParm

